@@ -15,8 +15,8 @@
 """Utility methods for objects"""
 
 import ast
-import six
 
+from oslo_serialization import jsonutils
 from oslo_versionedobjects import fields
 
 
@@ -32,7 +32,14 @@ NonNegativeIntegerField = fields.NonNegativeIntegerField
 ObjectField = fields.ObjectField
 StringField = fields.StringField
 UnspecifiedDefault = fields.UnspecifiedDefault
-UUIDField = fields.UUIDField
+
+
+class UUIDField(fields.UUIDField):
+    def coerce(self, obj, attr, value):
+        if value is None or value == "":
+            return self._null(obj, attr)
+        else:
+            return self._type.coerce(obj, attr, value)
 
 
 class Numeric(fields.FieldType):
@@ -52,10 +59,14 @@ class DictField(fields.AutoTypedField):
     AUTO_TYPE = fields.Dict(fields.FieldType())
 
 
+class ListOfUUIDsField(fields.AutoTypedField):
+    AUTO_TYPE = fields.List(fields.UUID())
+
+
 class FlexibleDict(fields.FieldType):
     @staticmethod
     def coerce(obj, attr, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             value = ast.literal_eval(value)
         return dict(value)
 
@@ -75,7 +86,7 @@ class FlexibleDictField(fields.AutoTypedField):
 class FlexibleListOfDict(fields.FieldType):
     @staticmethod
     def coerce(obj, attr, value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             value = ast.literal_eval(value)
         return list(value)
 
@@ -92,8 +103,26 @@ class FlexibleListOfDictField(fields.AutoTypedField):
         super(FlexibleListOfDictField, self)._null(obj, attr)
 
 
+class Json(fields.FieldType):
+    def coerce(self, obj, attr, value):
+        if isinstance(value, str):
+            loaded = jsonutils.loads(value)
+            return loaded
+        return value
+
+    def from_primitive(self, obj, attr, value):
+        return self.coerce(obj, attr, value)
+
+    def to_primitive(self, obj, attr, value):
+        return jsonutils.dumps(value)
+
+
+class JsonField(fields.AutoTypedField):
+    AUTO_TYPE = Json()
+
 # ### Notification fields ### #
 
+
 class BaseWatcherEnum(Enum):
 
     ALL = ()
@@ -130,7 +159,10 @@ class NotificationAction(BaseWatcherEnum):
     PLANNER = 'planner'
     EXECUTION = 'execution'
 
-    ALL = (CREATE, UPDATE, EXCEPTION, DELETE, STRATEGY, PLANNER, EXECUTION)
+    CANCEL = 'cancel'
+
+    ALL = (CREATE, UPDATE, EXCEPTION, DELETE, STRATEGY, PLANNER, EXECUTION,
+           CANCEL)
 
 
 class NotificationPriorityField(BaseEnumField):
